"""Module to handle identifiers."""

import zoia.ids.arxiv as arxiv  # noqa: F401
import zoia.ids.classification as classification  # noqa: F401
import zoia.ids.doi as doi  # noqa: F401
import zoia.ids.isbn as isbn  # noqa: F401
import zoia.ids.pdf as pdf  # noqa: F401
